/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL08 for C using RDMSQL */
/* This example shows how to use transactions with SQL commands.  A database
 * is created and then data is inserted within a transaction, which is
 * committed and then the data is read out and displayed.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *      rsqlTransStart
 *      rsqlTransCommit
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDMSQL call.
 */
static void drop_database(
    HCONN       hDbc,
    const char *name)
{
    HSTMT hStmt;
    char  commandText[80];

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    sprintf(commandText, "DROP DATABASE %s", name);
    (void) rsqlExecDirect(hStmt, commandText);
    (void) rsqlExecDirect(hStmt, "COMMIT");

    (void) rsqlFreeStmt(hStmt);
}

static void create_this_database(
    HCONN       hDbc,
    const char *name,
    const char *cmds[],
    uint32_t    cmd_length)
{
    RSQL_ERRCODE rsqlCode;
    HSTMT        hStmt;
    char         commandText[80];
    uint16_t     ii;

    (void) rsqlAllocStmt(hDbc, &hStmt);

    sprintf(commandText, "CREATE DATABASE %s", name);
    rsqlCode = rsqlExecDirect(hStmt, commandText);

    for (ii = 0; ii < cmd_length; ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the %s database\n", name);

    (void) rsqlExecDirect(hStmt, "COMMIT");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, name);

    (void) rsqlFreeStmt(hStmt);
}

static void create_databases(
    HCONN hDbc)
{
    const char *cmds[] = {
        "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)"
    };

    /* Drop any existing copies of the databases. */
    drop_database(hDbc, "SQL08_db");

    /* Attempt to create the databases */
    create_this_database(hDbc, "SQL08_db", cmds, 1);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    int16_t           ii;

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);

    create_databases(hDbc);

    /* Specify which databases to use */
    rsqlCode = rsqlOpenDB(hDbc, "SQL08_db", "x");

    /* Start a new transaction */    
    (void) rsqlTransStart(hDbc, "SQL08_trans");

    /* Create a new statement */
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Insert three rows into the table */
    for (ii = 1; ii < 4; ii++)
    {
        char commandText[80];

        sprintf(commandText, "INSERT INTO Table1 VALUES %hd", ii);
        rsqlCode = rsqlExecDirect(hStmt, commandText);
    }

    (void) rsqlTransCommit(hDbc);

    rsqlCode = rsqlExecDirect(hStmt, "SELECT id FROM Table1");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%d\n", result[0].vt.lv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc, "SQL08_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

